/*****************************************************************************

  global_elements.hpp
  
   This file belongs to the KisTA library
   All rights reserved by the authors (until further License definition)

  Author: F. Herrera
  Institution: KTH
  Deparment:   Electronic Systems
  Date: 2013 May

 *****************************************************************************/

#ifndef GLOBAL_ELEMENTS_HPP
#define GLOBAL_ELEMENTS_HPP

#include "types.hpp"
#include "utils.hpp"

namespace kista {

extern sc_time global_sim_time;
extern sketch_report_t sketch_report;

extern taskset_t tasks;
// This is a global hash table for fast access to task info from
// the process handler, and by name:
// These global structures are filled by the different schedulers
extern tasks_info_by_phandle_t	task_info_by_phandle;
extern tasks_info_by_name_t		task_info_by_name;
extern tasks_info_by_name_t		env_tasks_by_name;

extern system_level_connection_set_t sys_conn;
// This "enforces" or assumes that even if there are different types of system-level channels, 
// the names associated for their logic link identifier is unique
extern system_level_connection_set_by_name_t sys_conn_by_name;

extern system_level_connection_type_set_t sys_conn_types;

extern sched_set_t scheds;
extern sched_set_by_name_t scheds_by_name;

extern pe_set_t pes;
// Global hash table for fast access to processing element info by name
// it is updated by each new PE created
extern PE_by_name_t PEref_by_name;

extern phy_commres_set_t phy_commres_set;
extern phy_comm_res_by_name_t phy_commres_by_name;

extern tdma_bus_set_t tdma_buses;

// global structures to let hooking of communication with kista-xml executable

// for the moment, only a finite set of channel types supported by the XML interface
extern fifo_buffer_int_set_t     fifo_buffer_int;
extern fifo_buffer_uint_set_t    fifo_buffer_uint;
extern fifo_buffer_short_set_t   fifo_buffer_short;
extern fifo_buffer_ushort_set_t  fifo_buffer_ushort;
extern fifo_buffer_float_set_t	 fifo_buffer_float;
extern fifo_buffer_double_set_t  fifo_buffer_double;
extern fifo_buffer_char_set_t    fifo_buffer_char;
extern fifo_buffer_voidp_set_t   fifo_buffer_voidp;

// By default, KisTA creates a functionality monitor
extern validator_t *functionality_monitor;

} // namespace kista

#endif



