/*****************************************************************************

  kista_decl.hpp
  
   This file belongs to the KisTA library
   All rights reserved by the authors (until further License definition)

  Author: F. Herrera
  Institution: KTH
  Deparment:   Electronic Systems
  Date: 2013 January

 *****************************************************************************/

#ifndef KISTA_DECL_HPP
#define KISTA_DECL_HPP

namespace kista {


// disptaching policies

enum scheduler_policy_t {
                         STATIC_SCHEDULING = 0,		// dispatch is established by a list which fixes the schedule and repeates for ever
                           // dynamic scheduler policies
                         RANDOM_SCHEDULING = 1,			// selects one random task among available ones
                         PSEUDO_RANDOM_SCHEDULING = 2,  // selects one rando task among available ones following a random and reproducible sequence
                         CYCLE_EXECUTIVE = 3,         // dispatch is performed in a cyclical manner
                                                      // (CPU granted to the next task ready in the list, where the list order is "random"
                                                      // (in kista implementation, defined by the assignation order)
                         STATIC_PRIORITIES = 4,       // dispatch relies on static priorities
															// STATIC PRIORITIES use, by default USER priorities for the ordering
															// set_rate_monotonic, and set_deadline_monotonic methods change and overide user priorities
                         DYNAMIC_PRIORITIES = 5       // dispatch relies on dynamic priorities by updating a priority list
                         };

enum static_priorities_policy_t {
						USER_PRIORITIES = 0,
						DEADLINE_MONOTONIC = 1,
						RATE_MONOTONIC = 2,
						DENSITY_MONOTONIC = 3
};

enum dynamic_priorities_policy_t {
	                     EARLIEST_DEADLINE_FIRST = 0,  
	                     USER_DYNAMIC_PRIORITY_POLICY =1
};

enum scheduler_triggering_policy_t {
						PREEMPTIVE = 0,
						COOPERATIVE= 1,
						NON_PREEMPTIVE = 2
};

typedef unsigned char task_state_t;
typedef unsigned char sched_state_t;


// TASK STATES:
// Active states
#define EXECUTING                'E'
#define WAIT_PERIOD              'W'
#define WAIT_COMM_SYNC           'C'
// Inactive states
#define READY                    'R'
#define INACTIVE_WAIT_PERIOD     'I'
#define INACTIVE_WAIT_COMM_SYNC  'X'

#define FINISHED		         'F'

// Meaning for user tasks:
//   Active states:
//    E: Executing over any processing resource
//    W: Waiting for a synchronization event (for communication or for periodical event), while holding (thus wasting) some processing resource
//    R: Ready to execute, but no processing resource granted
//    I: Waiting for a synchronization event (for communication or for periodical event), without processing resource granted


// SCHEDULER STATES:
#define SCHEDULING               'S'
#define INNACTIVE		         'I'
//#define WAIT_COMM_SYNC         'C'
// 'C' state for scheduler can be used when supporting synchronization among schedulers

// Meaning for scheduler tasks:
//    E: Executing, so scheduling
//    R: Ready to execute, so no scheduling because an user task is executing (or in active waiting), thus consuming the processing resource scheduled
//    W: Waiting due to a communication (context switch) or to a synchronization (with other scheduler)

} // end namespace kista

#endif

